from typing import Any
# from .IUsersTable import IUsersTable


class IDatabase:
    users: Any

    def connect(self): ...
    def execute(self, *args, **kwargs) -> Any: ...
    def commit(self): ...
    def close(self): ...

